
from Velkoz.tentacles import (ChampionMasteryApi)

from Velkoz.eye import (ChampionMastery)


def _get_all_masteries_by_puuid(service, puuid:str, region:str = 'euw1'):
    parameters = {'puuid':puuid, 'region':region}
    championMasteryDto = ChampionMasteryApi.get_all_masteries(service, parameters)
    return championMasteryDto


def _get_champion_mastery(service, puuid:str, championId:int, region:str = 'euw1'):
    parameters = {'puuid':puuid, 'championId':championId, 'region':region}
    championMasteryDto = ChampionMasteryApi.get_mastery(service, parameters)
    championMastery = ChampionMastery(championMasteryDto)
    return championMastery


def _get_top_masteries(service, puuid:str, query:dict = {}, region:str = 'euw1'):
    parameters = {'puuid':puuid, 'region':region, 'query': query}
    championMasteryDto = ChampionMasteryApi.get_top_masteries(service, parameters)
    return championMasteryDto

def _get_masteryscore(service, puuid:str, region:str = 'euw1'):
    parameters = {'puuid':puuid, 'region':region}
    masteryscore = ChampionMasteryApi.get_masteryscore(service, parameters)
    return masteryscore






from Velkoz.tentacles import (MatchApi)

from Velkoz.eye import Match


#############
# Get Match #
#############

def _get_matchDto_by_parameters(service, parameters:dict):
    matchDto = MatchApi.get_matchDto(service, parameters)
    return matchDto






from .velkoz import(
        set_api_key,
        set_ddragon,
        get_account,
        get_summoner,
        get_match_list,
        get_match,
        get_all_masteries,
        get_champion_mastery,
        get_top_masteries,
        get_masteryscore,
        get_champion,
        get_champ_image,
        get_champName_by_key

        )

from .settings import *



from .tentacles import (
        AccountApi,
        ChampionMasteryApi,
        RiotApiService,
        api_key,
        MatchApi,
        SummonerApi
        )

from .eye import (
        Summoner,
        Match,
        Participant,
        Account
        )
from .data import(
        DdragonRequest,
        Champion,
        RiotJson,
        Skins

        )



from Velkoz.data import (Champion, RiotJson)


def _get_splash(champ:Champion, number:int):
    splash = champ.skins.get_splash(number)
    return splash


def _get_loading(champ:Champion, number:int):
    loading = champ.skins.get_loading(number)
    return loading




import requests



class RiotApiService():
    #This class needs error handling
    def __init__(
        self,
        api_key:str
    ):
        self.api_key = api_key
        self._headers = {"X-Riot-Token":api_key}
    
    def _validate_parameters(
        self,
        parameters:dict,
        necessary_parameters:dict = {}, #ONE OF THESE
        needed_parameters:dict = {} #ALL OF THESE

    ):
        #Returns True if parameters has at least one of the required parameters
        #If it doesn't return False.
        #Prevents a lot of errors that might occur due to incorrect syntax
        necessary = False
        needed = True
        for param in necessary_parameters:
            if param in parameters:
                necessary = True
        if necessary_parameters == {}: necessary = True
        for param in needed_parameters:
            if param not in parameters:
                needed = False
        if needed_parameters == {}: needed = True
        return (necessary and needed)
    

    def _set_defaults(
        self,
        parameters:dict
    ):
        # transforms the 'query' element of parameters from a dictionary
        # to a string for the url
        query = parameters['query']
        query_params = 'ids?'

        if 'startTime' not in query:
            query_params += ''
        else:
            value = query['startTime']
            query_params += f'startTime={value}&'

        if 'endTime' not in query:
            query_params += ''
        else:
            value = query['endTime']
            query_params += f'endTime={value}&'

        if 'queue' not in query:
            query_params += ''
        else:
            value = query['queue']
            query_params += f'queue={value}&'

        if 'type' not in query:
            query_params += ''
        else:
            value = query['type']
            query_params += f'type={value}&'
        
        if 'start' not in query:
            query_params += 'start=0&'
        else:
            value = query['start']
            query_params += f'start={value}&'

        if 'count' not in query:
            query_params += f'count=20'
        else:
            value = query['count']
            query_params += f'count={value}'

        parameters['query'] = query_params
        return parameters


    def _get(
        self,
        url:str,
        parameters: dict
    ):
        request = RiotApiRequest(
            service=self,
            url=url,
            parameters=parameters
        )
        try:
            return request()
        except Exception as error:
            raise Exception (error)
    
    def _handle_error(
        self,
        error
    ):
        if error == 'Incorrect Parameters':
            return f'Detected error: {error}.'
        else:
            raise Exception ('Raised exception: {error}')



class RiotApiRequest(object):
    #This class needs error handling based on the riot api error codes
    def __init__(
        self,
        service: RiotApiService,
        url:str,
        parameters:str
    ):
        self.service = service
        self.url = url
        self.parameters = parameters


    def __call__(self):
        try:
            header = self.service._headers
            response = requests.get(self.url, headers=header)
            if response.status_code == 200:
                return response.json()
            else:
                return self._handle_errorcode(response.status_code)

        except requests.exceptions.InvalidHeader as error:
            print(f'Invalid Header')
            return(self.service._headers)
        except requests.exceptions.HTTPError as errh:
            print(f'HTTPError - {errh.args[0]}')
            return self._retry_request_on_error(errh)
    
    def _retry_request_on_error(self, errh):
        raise Exception (errh)
        #since I do not know how it reports the error it will stay unhandled for now
        
    def _handle_errorcode(self, code):
        if code == 400:
            raise Exception (f'Request failed with status code {code}: Bad request')
        elif code == 403:
            raise Exception (f'Request failed with status code {code}: Forbidden')
        elif code == 429:
            raise Exception (f'Request failed with status code {code}: Rate Limit Exceeded')

        


if __name__ == '__main__':
    pass



from .common import (
    RiotJson,
    DdragonRequest

        )


class Champion(RiotJson):

    def __init__(self, **kwargs):
        self.request = kwargs.get('ddragon', None)
        dto = kwargs.get('dto', None)
        RiotJson.__init__(self, dto)
        self.champ = kwargs.get('champion', None)
        self.championDto = self.data.get(self.champ, None)
        try:
            self._general(self.championDto)
        except Exception as error:
            print(self.champ)
   
    def _general(self, Dto:dict):
        self.id = Dto.get('id', self.champ)
        self.key = Dto.get('key', None)
        self.name = Dto.get('name', None)
        self.title = Dto.get('title', None)

        self.image = Dto.get('image', None)

        #func to skins
        self._set_skins(Dto.get('skins', None))
        #func to tips
        self.tags = Dto.get('tags', None)
        self.partype = Dto.get('partype', None) #Resource?
        self.info = Dto.get('info', None)
        self.stats = Dto.get('stats', None)
        self.spells = Dto.get('spells', None)
        self.passive = Dto.get('passive', None)
        self.recommended = Dto.get('recommended', None) #what even is this

    def _set_skins(self, skinsArray):
        self.skins = Skins(champion=self.champ, request=self.request, skinsArray=skinsArray) 

    

class Skins(Champion):
   
    def __init__(self, **kwargs):
        self.request = kwargs.get('request',None)
        self.champion= kwargs.get('champion',None)
        self.all = kwargs.get('skinsArray', [])
        self.skinNum = len(self.all)
       
    def _handleIndexError(self):
        raise Exception ('This champion does not have that many skins :(')


    def _get_img(self, get_type, number):
        try:
            skinNumber = self.all[number].get('num', 0)
            return self.request._get_champion_image(get_type=get_type, champion=self.champion, num=skinNumber)
        except IndexError:
            self._handleIndexError()

    def get_splash(self,number):
        return self._get_img('splash',number)

    def get_loading(self,number):
        return self._get_img('loading',number) 

    def get_centered(self,number):
        return self._get_img('centered',number)

    def get_tile(self,number):
        return self._get_img('tiles',number)
    
    def skin(self, number):
        try:
            return self.all[number]
        except IndexError:
            self._handleIndexError()


    def __str__(self):
        return (f'{self.champion} has {self.skinNum - 1} skins')
        






#useless?
class ChampionMastery():
    

    def __init__(self, kwargs):
        self.ChampionMasteryDto = kwargs
        self.chestGranted = kwargs.get('chestGranted' ,None)
        self.championId = kwargs.get('championId' ,None)
        self.lastPlayTime = kwargs.get('lastPlayTime', None)
        self.championLevel = kwargs.get('championLevel', None)
        self.championPoints = kwargs.get('championPoints', None)
        self.tokensEarned = kwargs.get('tokensEarned', None)

    def get(self, parameter):
        got = self.ChampionMasteryDto.get(parameter, None)





class Account():

    def __init__(self, accountDto:dict):
        self.accountDto = accountDto
        self.puuid = accountDto['puuid']
        if 'gameName' in accountDto:
            self.gameName = accountDto['gameName']
        if 'tagLine' in accountDto:
            self.tagLine = accountDto['tagLine']
            try:
                self.riotId = self.gameName + '#' + self.tagLine
            except:
                self.riotId = ''

    def save(self, author):
        print('saved account')



from .common import RiotApiService

class MatchApi(RiotApiService):

    def get_match_list(
        self,
        parameters:dict
    ):
        necessary_parameters = {'puuid'}
        needed_parameters = {'routing', 'query'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
 
        parameters = self._set_defaults(parameters)
        routing = parameters['routing'].lower()
        puuid = parameters['puuid']
        query = parameters['query']
        try:
            url = f'https://{routing}.api.riotgames.com/lol/match/v5/matches/by-puuid/{puuid}/{query}'

            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error


    # Will have to handle all the info in match accordingly
    def get_matchDto(
        self,
        parameters:dict
    ):
        necessary_parameters = {'matchId'}
        needed_parameters = {'routing'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
 

        routing = parameters['routing'].lower()
        matchId = parameters['matchId']

        try:
            url = f'https://{routing}.api.riotgames.com/lol/match/v5/matches/{matchId}'
            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error







from .common import RiotApiService

class ChampionMasteryApi(RiotApiService):

    def get_all_masteries(
        self,
        parameters:dict
    ):
        necessary_parameters = {}
        needed_parameters = {'region', 'puuid'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
        
        region = parameters['region'].lower()
        puuid = parameters['puuid']
        try:
            url =f'https://{region}.api.riotgames.com/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}' 
            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error

    
    def get_mastery(
        self,
        parameters:dict
    ):
        necessary_parameters={}
        needed_parameters={'puuid', 'championId', 'region'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
        region = parameters['region'].lower()
        puuid = parameters['puuid']
        championId = parameters['championId']
        try:
            url = f'https://{region}.api.riotgames.com/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/by-champion/{championId}'

            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error

    def get_top_masteries(
        self,
        parameters:dict
    ):
        necessary_parameters = {}
        needed_parameters = {'puuid', 'region', 'query'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
        
        query = parameters['query']
        if 'count' not in query:
            query_params = ''
        else:
            value = query['count']
            query_params = f'?count={value}'

        region = parameters['region'].lower()
        puuid = parameters['puuid']
        try:
            url = f'https://{region}.api.riotgames.com/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/top{query_params}'
 
            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error

    def get_masteryscore(
        self,
        parameters:dict
    ):
        necessary_parameters = {}
        needed_parameters = {'puuid', 'region'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        
        if not are_param_valid: return self._handle_error('Incorrect Parameters')
       
        puuid = parameters['puuid']
        region = parameters['region'].lower()
        try:
            url = f'https://{region}.api.riotgames.com/lol/champion-mastery/v4/scores/by-puuid/{puuid}'

            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error








import json
from PIL import Image as ImageLoader
from PIL.Image import Image
from io import BytesIO


version = '14.7.1'
language = 'en_US'

class DdragonRequest():

    def __init__(self, **kwargs):
        self.ddragon_path = kwargs.get('ddragon_path', None)
  

    def _get_data(self, **kwargs):
        get_type = kwargs.get('get_type', None)

        match get_type:
            case 'champion':
                champ = kwargs.get('champion', None)
                path = f'{self.ddragon_path}/{version}/data/{language}/champion/{champ}.json'
                file=open(path,'r')
                champion_data = json.load(file)
                return champion_data
            
            case 'item':
                path = f'{self.ddragon_path}/{version}/data/{language}/item.json'
                file=open(path,'r')
                item_data = json.load(file)
                return item_data
            
            case _:
                raise Exception ('Invalid data type')
            

    def _get_champion_image(self, **kwargs):
        get_type = kwargs.get('get_type', 'splash')
        num = kwargs.get('num', 0)
        champ = kwargs.get('champion', 'Aatrox')
        path = f'{self.ddragon_path}/img/champion/{get_type}/{champ}_{num}.jpg'
        with open(path, 'rb') as image:
            f = image.read()
            b = bytearray(f)
        return ImageLoader.open(BytesIO(b))
        # return data


class RiotJson():

    def __init__(self, Dto):
        self.type = Dto.get('type', None)
        self.format = Dto.get('format', None)
        self.version = Dto.get('version', None)
        self.data = Dto.get('data', None)


    



        



from .tentacles import ( #dont forget the dots
    AccountApi,
    SummonerApi,
    api_key,
    RiotApiService,
    MatchApi
    )

from .eye import ( #dont forget the dots
    Summoner,
    Match,
    Participant,
    Account
        )

from .data import(
    DdragonRequest,
    Champion,
    RiotJson
        )

from .supp import ( #dont forget the dots
    summoner as summ,
    account as acc,
    match as mat,
    champion_mastery as mastery

        ) 
from .settings import (all_champion_id)

#CHANGE THIS
#ERROR HANDLING FOR INVALID API_KEY

def set_api_key(api_key:str):
    global service
    service = RiotApiService(api_key)

def set_ddragon(path:str):
    global ddragon
    ddragon = DdragonRequest(ddragon_path=path)

def _initialize_service(api_key=api_key):
    return RiotApiService(api_key) # Service should be initiated in the needed file




def _handle_missing_parameters(request:str):

    raise Exception (f'Error fulfilling request {request}. Make sure you are passing the correct arguments')




#for now only used in champion_mastery functions
def _get_puuid(puuid:str, riotId:str, routing:str = 'europe'):

    if puuid != '\0':
        return puuid
    elif riotId != '\0':
        account = get_account(riotId=riotId, routing=routing)
        return account.puuid
    else:
        _handle_missing_parameters('_get_puuid')



############
# Callable #
############



'''
Function to get account
Usage: Must be provided a riotId or puuid. If both are provided it will default to using riotId. Routing parameter defaults to 'europe' so it is optional, but it's recommended to provide it with the nearest routing to you.
'''
def get_account(riotId:str = '\0', puuid:str = '\0', routing:str = 'europe'):

    parameters = {}
    if riotId != '\0':
        parameters['riotId'] = riotId
    elif puuid != '\0':
        parameters['puuid'] = puuid
    else:
        raise Exception ('Error getting account. Please provide a riotId or puuid')

    parameters['routing'] = routing
    account = acc._get_account_by_parameters(service, parameters)
    return account

'''
Function to get summoner
Usage: Must be provided with a riotId, summonerName or summonerId. Having priority in the order they were mentioned here. Region parameter defualts to 'euw1', but should be changed if the summoner you are looking for is not from the euw server.
'''
def get_summoner(puuid:str = '\0', riotId:str = '\0', summonerName:str = '\0', summonerId:str = 0, region:str = 'euw1'):
    
    if riotId != '\0':
        account = acc._get_account_by_riotId(service, riotId)
        parameters = {'puuid' : account.puuid}
    elif puuid != '\0':
        parameters = {'puuid': puuid}

    elif summonerName != '\0':
        parameters = {'summonerName': summonerName}

    elif summonerId != 0:
        parameters = {'summonerId' : summonerId}

    else:
        raise Exception ('Error getting summoner. Please provide a riotId, summonerName or summonerId')

    parameters['region'] = region
    summoner = summ._get_summoner_by_parameters(service, parameters)
    return summoner



'''
Function to get a list of matches given a riotId or puuid
needs more arguments, possibly use args to get the query
returns an array with the match ids
'''
def get_match_list(riotId:str = '\0', puuid:str = '\0', routing:str = 'europe', count:int = 20):
    
    if riotId != '\0':
        account = acc._get_account_by_riotId(service, riotId) 
        parameters = {'puuid': account.puuid}

    elif puuid != '\0':
        parameters = {'puuid': puuid}

    parameters['routing'] = routing
    query = {'count' : count}
    parameters['query'] = query
    match_list = MatchApi.get_match_list(service, parameters)
    return match_list
        
'''
Function to get a Match by a matchId
Usage: Must be provided a matchId. Routing parameter is not necessary since it defaults to 'europe', but must be changed if the match is not from european servers.
'''

def get_match(matchId:str, routing:str = 'europe'):

    parameters = {'matchId' : matchId, 'routing': routing}
    matchDto = mat._get_matchDto_by_parameters(service, parameters)
    match = Match(matchDto)
    return match





####################
# Champion Mastery 
####################

def get_all_masteries(puuid:str = '\0', riotId:str = '\0', region = 'euw1', routing = 'europe'):
    
    new_puuid = _get_puuid(puuid, riotId, routing)

    return mastery._get_all_masteries_by_puuid(service, new_puuid, region = region)


def get_champion_mastery(puuid:str = '\0', riotId:str = '\0', championId:int = 1 ,region:str = 'euw1', routing:str = 'europe'):

    new_puuid = _get_puuid(puuid, riotId, routing)


    return mastery._get_champion_mastery(service=service, puuid = new_puuid, championId = championId,region = region)

def get_top_masteries(puuid:str = '\0', riotId:str = '\0', region:str = 'euw1', routing:str = 'europe', **kwargs):

    new_puuid = _get_puuid(puuid, riotId, routing)

    query = {}
    for key, value in kwargs.items():
        query[key] = value

    return mastery._get_top_masteries(service, puuid=puuid, query=query, region=region)


def get_masteryscore(puuid:str='\0', riotId:str='\0',region:str='euw1', routing:str='europe'):

    new_puuid = _get_puuid(puuid, riotId, routing)

    return mastery._get_masteryscore(service, puuid=new_puuid, region=region)

############
# Champion #
############


def get_champion(champion:str):
    champDto = ddragon._get_data(get_type='champion', champion=champion)
    champ = Champion(ddragon=ddragon, dto=champDto, champion=champion)
    return champ

def get_champ_image(champion:str, **kwargs):
    champ = get_champion(champion)

    img_type = kwargs.get('image', 'splash')
    number = kwargs.get('skin', 0)
    match img_type:
        case 'splash':
            image = champ.skins.get_splash(number)

        case 'loading':
            image = champ.skins.get_loading(number)

        case 'centered':
            image = champ.skins.get_centered(number)

        case 'get_tile':
            image = champ.skins.get_tile(number)

        case _:
            raise Exception ('That type of image doesnt exist')

    return image


def get_champName_by_key(champId:int):
    return all_champion_id.get(champId, None)



#############
# DEBUGGING #
#############

if __name__ == '__main__':
    pass 




# DO NOT FORGET THE DOTS!!!!!!
from Velkoz.tentacles import (SummonerApi, RiotApiService, api_key) 

from Velkoz.eye import Summoner

#service = RiotApiService(api_key)


def _get_summonerDto_by_account(service, account:dict, region:str = 'euw1'):
    parameters = {'puuid' : account['puuid'], 'region': region}
    summonerDto = SummonerApi.get_summoner(service, parameters)
    return summonerDto


def _get_summoner_by_account(service, account:dict, region:str = 'euw1'):
    parameters = {'puuid': account['puuid'], 'region' : region}
    summonerDto = SummonerApi.get_summoner(service, parameters)
    return Summoner(summonerDto)

def _get_summoner_by_summonerName(service, summonerName:str, region:str = 'euw1'):
    parameters = {'summonerName': summonerName, 'region': region}
    summonerDto = SummonerApi.get_summoner(service,parameters)
    return Summoner(summonerDto)

def _get_summoner_by_summonerId(service, summonerId:str, region:str = 'euw1'):
    parameters = {'summonerId': summonerId, 'region': region}
    summonerDto = SummonerApi.get_summoner(service, parameters)
    return Summoner(summonerDto)


def _get_summoner_by_parameters(service, parameters):
    summonerDto = SummonerApi.get_summoner(service, parameters)
    return Summoner(summonerDto)



def _request_on_summoner_by_riotId(riotId:str, routing:str = 'europe',region:str = 'euw1', request={'summonerLevel'}):
    account = _get_account_by_riotId(riotId, routing) 
    summonerDto = _get_summonerDto_by_account(account, region)
    summoner = Summoner(summonerDto)
    summoner_data = summoner.request({'data':request})
    return summoner_data






from setuptools import setup, find_packages

 #python setup.py sdist bdist_wheel
VERSION = '0.0.52' 
DESCRIPTION = 'Velkoz Riot Api package'
LONG_DESCRIPTION = 'Package that uses RiotApi to get information.\n Made to gain experience so it is not something as advanced as the other packages that do the same. Would not recommend.'

# Setting up
setup(
        name="Velkoz", 
        version=VERSION,
        author="Nabattis",
        author_email="<ferreirafernando6205@gmail.com>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['requests','pillow'], # add any additional packages that 
        # needs to be installed along with your package. Eg: 'caer'
        
        keywords=['python', 'riotapi', 'api', 'velkoz'],
        classifiers= [
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)



from .common import RiotApiService

class SummonerApi(RiotApiService):

    def get_summoner(
        self,
        parameters:dict
    ):
        necessary_parameters = {'puuid', 'summonerName', 'accountId'}
        needed_parameters = {'region'}
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        if not are_param_valid: return self._handle_error('Incorrect Parameters')

        region = parameters['region'].lower()
        
        # Summonerids are not supported because of potential blacklists
        if 'puuid' in parameters:
           puuid = parameters['puuid']
           url = f'https://{region}.api.riotgames.com/lol/summoner/v4/summoners/by-puuid/{puuid}'
        
        elif 'summonerName' in parameters:
            summonerName = parameters['summonerName']
            url = f'https://{region}.api.riotgames.com/lol/summoner/v4/summoners/by-name/{summonerName}'
        
        elif 'accountId' in parameters:
            accountId = parameters['accountId']
            url = f'https://{region}.api.riotgames.com/lol/summoner/v4/summoners/by-account/{accountId}' 


        else:
            return 'HOW DID WE EVEN GET HERE'


        try:
            return self._get(url=url, parameters=parameters)

        except requests.exceptions.APINotFoundError as error:
            return error





from .account import AccountApi
from .champion_mastery import ChampionMasteryApi 
from .common import RiotApiService
from .config import api_key
from .match import MatchApi
from .summoner import SummonerApi



from .common import RiotApiService

class AccountApi(RiotApiService):

    def get_account(
            self, parameters:dict):
        necessary_parameters = {'puuid', 'riotId'}
        needed_parameters = {'routing'}
        #Checks if the given parameters are valid for this function
        are_param_valid = self._validate_parameters(parameters, necessary_parameters, needed_parameters)
        if not are_param_valid: return self._handle_error('Incorrect Parameters')

        if 'riotId' in parameters:
            riotid = parameters['riotId'].split('#', 1)
            routing = parameters['routing']
            url = f'https://{routing.lower()}.api.riotgames.com/riot/account/v1/accounts/by-riot-id/{riotid[0]}/{riotid[1]}'
        elif 'puuid' in parameters:
            puuid = parameters['puuid']
            routing = parameters['routing']
            url = f'https://{routing.lower()}.api.riotgames.com/riot/account/v1/accounts/by-puuid/{puuid}'
        else:
            raise Exception ('HOW DID WE GET HERE LMAO')

        try:
            return self._get(url=url, parameters=parameters)
        except requests.exceptions.APINotFoundError as error:
            return error





class Summoner():
    
    def __init__(self, summonerDto:dict):
        self.summonerDto = summonerDto
        self.id = summonerDto['id']
        self.accountId = summonerDto['accountId']
        self.puuid = summonerDto['puuid']
        self.name = summonerDto.get('name', None)
        self.profileIcon = summonerDto['profileIconId']
        self.revisionDate = summonerDto['revisionDate']
        self.level = summonerDto['summonerLevel']

    def _save_summoner(self, author):
        print(f'Summoner saved with author {author}(actually did nothing)')


    def _validate_data_request(self,data_request:dict):
        for item in data_request:
            if item not in self.summonerDto:
                return 'Request for invalid data'
            
    #Gets specific data from summonerDto
    def _get_data(self,data_request:dict):
        
        self._validate_data_request(data_request)
        
        answer_data = []
        for item in data_request:
            answer_data.append(self.summonerDto[item])
        return answer_data


    def request(self, requests:dict):
        to_return = 'Requests have been met'
        for request in requests:
            if request == 'data':
                to_return = self._get_data(requests['data'])
            elif request == 'save':
                self._save_summoner(requests['save'])
            else:
                return 'invalid request'

        return to_return


if __name__ == '__main__':

    summonerDto ={'id': 'Cdp1KXJgqkSMIMwwhuxkv3_YfUfwNW_9n9gcwbE1mfkhrIc', 'accountId': 'xGsuWPAHONj6AuzPQk2sdjcWIuxnOoUE5qNXQHhDjUnu5so', 'puuid': 'AD2f5GhQOeuMeJ_XAle6mfO1sPqrHeru0qGpri3vJGoS3bO22_iUWW2JtRP3CkefhPhtolDSvfUdhg', 'name': 'Zemiranda04', 'profileIconId': 5495, 'revisionDate': 1710790106920, 'summonerLevel': 121}
    
    requests = {'data':{'name', 'summonerLevel'}, 'save': 'diogo'}
    sumo = Summoner(summonerDto)
    print(answer)





class Match():

    def __init__(self, matchDto:dict):
        self.metadataDto = matchDto['metadata']
        self.infoDto = matchDto['info']
        self._define_metadata(self.metadataDto)
        self._define_info(self.infoDto)


    def _define_metadata(self, metadata:dict):
        self.version = metadata['dataVersion']
        self.matchId = metadata['matchId']
        self.participants = metadata['participants']

    def _define_info(self, kwargs):
        self.creation = kwargs.get('gameCreation', None)
        self.duration = kwargs.get('gameDuration', None)
        self.end = kwargs.get('gameEndTimestamp', None)
        self.gameId = kwargs.get('gameId', None)
        self.gameMode = kwargs.get('gameMode', None)
        self.start = kwargs.get('gameStartTimestamp', None)
        self.gameType = kwargs.get('gameType', None)
        self.mapId = kwargs.get('mapId',None)
        self.platformId = kwargs.get('platformId', None)
        self.queueId = kwargs.get('queueId', None)
        self.teams = kwargs.get('teams', None)
        self.tournamentCode = kwargs.get('tournamentCode', None)
        self.gameVersion = kwargs.get('gameVersion', None)
    
    def _get_participant_by_puuid(self, puuid:str):
        i = 0
        while (i<10):
            if self.participants[i] == puuid:
                return i
            else: i += 1
        raise Exception ('Participant not found.')


    def _get_participant_by_riotId(self, riotId:str):
        i = 0
        while (i < 10):
            riotName = self.infoDto['participants'][i].get('riotIdGameName', None)
            tagline = self.infoDto['participants'][i].get('riotIdTagline', None)
            if riotId == f'{riotName}#{tagline}': return i
            else: i += 1
        raise Exception ('Participant not found.')
    

    def get_participant(self, index:int = 0, puuid:str = '\0', riotId:str = '\0'):
        if puuid != '\0':
            i = self._get_participant_by_puuid(puuid)
        
        elif riotId != '\0':
            i = self._get_participant_by_riotId(riotId)

        else:
            #when index is provided 
            i = index
            if index < 0 or index > len(self.participants) + 1:
                raise Exception ('Invalid index')

        return Participant(self.infoDto['participants'][i])
    



class Participant():

    def __init__(self, kwargs):
        self.participantDto = kwargs
        self.assists = kwargs.get('assists', None)
        self.baronKills = kwargs.get('baronKills', None)
        self.bountyLevel = kwargs.get('bountyLevel', None)
        self.champExperience = kwargs.get('champExperience', None)
        self.champLevel = kwargs.get('champLevel', None)
        self.championId = kwargs.get('championId', None) #Prior to 11.4 could give errors
        self.championName = kwargs.get('championName', None)
        self.championTransform = kwargs.get('championTransform', None) #Only applicable to Kayn
        self.consumablesPurchased = kwargs.get('consumablesPurchased', None)
        self.damageDealtToBuildings = kwargs.get('damageDealtToBuildings', None)
        self.damageDealtToObjectives = kwargs.get('damageDealtToObjectives', None)
        self.damageDealtToTurrets = kwargs.get('damageDealtToTurrets',None)
        self.deaths = kwargs.get('deaths', None)
        self.detectorWardsPlaced = kwargs.get('detectorWardsPlaced', None)
        self.doubleKills = kwargs.get('doubleKills', None)
        self.dragonKills = kwargs.get('dragonKills', None)
        self.firstBloodKill = kwargs.get('firstBloodKill', None)
        self.firstTowerKill = kwargs.get('firstTowerKill', None)
        self.gameEndedInEarlySurrender = kwargs.get('gameEndedInEarlySurrender', None)
        self.gameEndedInSurrender = kwargs.get('gameEndedInSurrender', None)
        self.goldEarned = kwargs.get('goldEarned', None)
        self.goldSpent = kwargs.get('goldSpent', None)
        self.position = kwargs.get('teamPosition', None) #could also use individualPosition
        self.inhibitorKills = kwargs.get('inhibitorKills', None)
        self.inhibitorsLost = kwargs.get('inhibitorsLost', None)
        #Items to array
        i = 0
        items = []
        while i < 7:
            items.append(kwargs.get(f'item{i}', None))
            i += 1
        ##
        self.kills = kwargs.get('kills', None)
        self.lane = kwargs.get('lane', None)
        self.largestMultiKill = kwargs.get('largestMultiKill', None)
        self.longestTimeSpentLiving = kwargs.get('longestTimeSpentLiving', None)
        self.magicDamageDealtToChampions = kwargs.get('magicDamageDealtToChampions', None)
        self.magicDamageTaken = kwargs.get('magicDamageTaken', None)
        self.cs = kwargs.get('neutralMinionsKilled', None)
        self.objectivesStolen = kwargs.get('objectivesStolen', None)
        self.id = kwargs.get('participantId', None)
        self.pentaKills = kwargs.get('pentaKills', None)
        self.physicalDamageDealtToChampions = kwargs.get('physicalDamageDealtToChampions', None)
        self.physicalDamageTaken = kwargs.get('physicalDamageTaken', None)
        self.profileIcon = kwargs.get('profileIcon', None)
        self.puuid = kwargs.get('puuid', None)
        self.quadraKills = kwargs.get('quadraKills', None)
        try:
            self.riotId = (kwargs.get('riotIdGameName', None) + '#' + kwargs.get('riotIdTagline', None))
        except:
            self.riotIdName = kwargs.get('riotIdGameName', None)
            self.riotIdTagline = kwargs.get('riotIdTagline', None)
        self.role = kwargs.get('role', None)
        self.sightWardsBoughtInGame = kwargs.get('sightWardsBoughtInGame', None)
        self.Qcasts = kwargs.get('spell1Casts', None)
        self.Wcasts = kwargs.get('spell2Casts', None)
        self.Ecasts = kwargs.get('spell3Casts', None)
        self.Rcasts = kwargs.get('spell4Casts', None)
        self.Dcasts = kwargs.get('summoner1Casts', None)
        self.Fcasts = kwargs.get('summoner2Casts', None)
        self.Did = kwargs.get('summoner1Id', None)
        self.Fid = kwargs.get('summoner2Id', None)
        self.summonerLevel = kwargs.get('summonerLevel', None)
        self.summonerName = kwargs.get('summonerName', None)
        self.teamId = kwargs.get('teamId', None)
        self.timeCCingOthers = kwargs.get('timeCCingOthers', None)
        self.timePlayed = kwargs.get('timePlayed', None)
        self.totalDamageDealtToChampions = kwargs.get('totalDamageDealtToChampions', None)
        self.totalDamageTaken = kwargs.get('totalDamageTaken', None)
        self.totalHeal = kwargs.get('totalHeal', None)
        self.totalHealsOnTeammates = kwargs.get('totalHealsOnTeammates', None)
        self.totalTimeCCDealt = kwargs.get('totalTimeCCDealt', None)
        self.totalTimeSpentDead = kwargs.get('totalTimeSpentDead', None)
        self.tripleKills = kwargs.get('tripleKills', None)
        self.trueDamageDealtToChampions = kwargs.get('trueDamageDealtToChampions', None)
        self.unrealKills = kwargs.get('unrealKills', None)
        self.visionScore = kwargs.get('visionScore', None)
        self.wardsKilled = kwargs.get('wardsKilled', None)
        self.win = kwargs.get('win', None)
   
    
    #this function can be used to get any key of participantDto that has not been defined above
    def get(self, parameter:str): 
        return self.participantDto.get(parameter, None)















from Velkoz.tentacles import (
    AccountApi

        )

from Velkoz.eye import (
    Account
        )


###############
# Get Account #
###############


def _get_account_by_riotId(service, riotId:str, routing:str = 'europe'):
    parameters = {'riotId': riotId, 'routing' : routing}
    accountDto = AccountApi.get_account(service, parameters)
    account = Account(accountDto)
    return account

def _get_account_by_parameters(service, parameters:dict):
    accountDto = AccountApi.get_account(service, parameters)
    account = Account(accountDto)
    return account

